      subroutine bondlength( vibtype, requil, kvib, beta, length, vvib )
c     **************************************************************
c     *** computes the bond length for a given vibration type    ***
c     *** vibtype is the vibration type                          ***
c     *** requil is the equilibrium bond length                  ***
c     *** kvib is the force constant for the bond length         ***
c     *** length is the bond length returned by this subroutine  ***
c     *** beta is 1/kT                                           ***
c     *** vvib is the vibration energy for this bond             ***
c     *** M.G. Martin  2-4-98                                    ***
c     **************************************************************

      implicit none

      integer vibtype
      double precision length, bond, random, bf, vvib, beta, kvib
     &     , requil

      vvib = 0.0d0

      if ( kvib .gt. 0.1d0 ) then
c     --- random bond length from Boltzmann distribution ---
 107     bond = (0.2d0*random()+0.9d0)
         
c     --- correct for jacobian by dividing by the max^3
c     --- 1.331  = (1.1)^3, the equilibrium bond length cancels out
         bf = bond*bond*bond/(1.331d0)
         if ( random() .ge. bf ) goto 107
         bond = bond * requil
         
c     --- correct for the bond energy
         vvib = kvib * (bond-requil )**2
         bf = dexp ( -(vvib * beta) )
         if ( random() .ge. bf ) goto 107
         length = bond
      else
c     --- fixed bond length
         length = requil
      endif
      
      return

      end
