      subroutine cone(iinit,x,y,z,alpha,gamma,ux,uy,uz)
c     *******************************************************************
c     ** if iinit = 1 then it sets up the rotation matrix for the cone **
c     ** using x,y,z as a unit vector pointing in the +z direction     **
c     *******************************************************************
c     ** if iinit = 2 then it creates a unit vector that has an angle  **
c     ** of alpha from the +z direction (previous vector) and an angle **
c     ** of gamma (0,2Pi) around the cone circle and returns this as   **
c     ** ux,uy,uz                                                      **
c     *******************************************************************
c     ** if iinit = 3 then this computes gamma (0,2Pi) for a unit      **
c     ** vector ux,uy,uz with angle alpha from the -z direction        **
c     *******************************************************************
c     ** note that the rotation matrix is saved after each call so it  **
c     ** needs to be reset when you wish to use another cone           **
c     ** modified by M.G. Martin 3-06-98                               **
c     *******************************************************************

      implicit none
      include 'conver.inc'

      integer iinit,count
      double precision x,y,z,alpha,gamma,ux,uy,uz
     +     ,a11,a12,a13,a21,a22,a31,a32,a33
     +     ,sinthe,costhe,sinpsi,cospsi,singamma,cosgamma
     +     ,uxtemp,uytemp,uztemp,sinalph,cosalph

      save a11,a12,a13,a21,a22,a31,a32,a33,cosalph,sinalph

c      write(6,*) 'start CONE'

      if ( iinit .eq. 1 ) then      
c     --- setup the unit cone
c     --- assume x,y,z is a rotation of vector (0,0,1)
c       x = -dsin(theta)*dcos(psi)
c       y = dsin(theta)*dsin(psi)
c       z = dcos(theta)
c
        costhe=z
        sinthe = dsqrt(1.0d0 - costhe*costhe)
        if (sinthe .ne. 0.0d0) then
          sinpsi=(y)/sinthe
	  cospsi=(-x)/sinthe
        else
          sinpsi = 0.0d0
	  cospsi = 1.0d0
        endif

c rotation matrix
        a11 = cospsi*costhe
        a12 = -(sinpsi*costhe)
        a13 = sinthe
        a21 = sinpsi
        a22 = cospsi
        a31 = x
        a32 = y
        a33 = z

c       a11 = cos(psi) cos(theta) cos(phi) - sin (psi) sin(phi)
c       a12 =-sin(psi) cos(theta) cos(phi) - cos (psi) sin(phi)
c       a13 =          sin(theta) cos(phi) 
c       a21 = cos(psi) cos(theta) sin(phi) + sin (psi) cos(phi)
c       a22 =-sin(psi) cos(theta) sin(phi) + cos (psi) cos(phi)
c       a23 =   sin(theta) sin(phi)
c       a23 = 0
c       a31 = -cos(psi) sin(theta)
c       a32 = sin(psi)  sin (theta)
c       a33 = cos(theta)

      elseif ( iinit .eq. 2 ) then
c        --- find the vector on a cone:
c        --- alpha is the angle with the cone axis (theta)
         sinalph = dsin(alpha)
         cosalph = dcos(alpha)
c        --- gamma is now passed to cone - (phi)
         uztemp = -cosalph
         uytemp = sinalph*dcos(gamma)
         uxtemp = sinalph*dsin(gamma)
         ux = a11*uxtemp + a21*uytemp + a31*uztemp
         uy = a12*uxtemp + a22*uytemp + a32*uztemp
         uz = a13*uxtemp +              a33*uztemp

      elseif ( iinit .eq. 3 ) then
c        --- find gamma for a given unit vector on the cone

         uztemp = (ux - uy*(a21/a22) + uz*((a21*a12/a22)-a11)/a13 ) /
     &        ( a31 - (a11*a33)/(a13) + (a21*a12*a33)/(a22*a13) 
     &        - (a21*a32)/(a22) )
         uytemp = (uy - a32*uztemp - a12*( (uz - a33*uztemp)/a13 ) )/a22
         uxtemp = (uz - a33*uztemp)/a13

         sinalph = dsin(alpha)
         singamma = uxtemp/sinalph
         cosgamma = uytemp/sinalph

         count = 0
c        --- make sure that the precision screw-ups don't affect cosgamma
         if ( cosgamma .lt. -1.0d0 ) then
            cosgamma = -1.0d0
         elseif ( cosgamma .gt. 1.0d0 ) then
            cosgamma = 1.0d0
         endif

c        --- now need to find the gamma on [-Pi,Pi] that satisfies cos and sin
         gamma = dacos(cosgamma)
         if ( singamma .lt. 0.0d0 ) gamma = -gamma

      else
         write(6,*) 'iinit ',iinit
         stop 'non valid iinit in cone.f'
      endif
      
c      write(6,*) 'finish CONE'

      return
      end


