      subroutine config
 
c    *******************************************************************
c    ** performs a lengthconserving configurational bias move         **
c    ** for linear, branched, anisotropic, and explicit atom          **
c    ** molecules                                                     **
c    ** rewritten from old config and branch subroutines by           **
c    ** M.G. Martin 9-19-97                                           **
c    ** number of trial attempts starting at unit inb is stored in    **
c    **    bncb ( inb ).                                              **
c    ** number of successful generations of trial configuration is in **
c    **    bscb ( 1,inb ).                                            **
c    ** number of accepted trial configurations is in                 **
c    **    bscb ( 2,inb ).                                            **
c    *******************************************************************
 
      implicit none

      include 'control.inc'
      include 'coord.inc'
      include 'coord2.inc'
      include 'system.inc'
      include 'ensemble.inc'
      include 'cbmc.inc'
      include 'rosen.inc' 
      include 'inputdata.inc'
      include 'poten.inc'
      include 'neigh.inc'
      include 'parallel.inc'     

      logical lterm, ovrlap, ltors,lneighij

      integer i,j,iii,ibox,iunit,igrow,icbu,islen,imolty,iutry

      integer istt,iett,nchp1,ic,ncount,total,ip,imol

      double precision v,vintra,vinter,vext,velect,vewald,vtorold
     & ,vtornew,delen,deleo,vdum

      double precision dchain,random,rchain,wnlog,wolog,wdlog,wratio

      double precision vrecipn,vrecipo,pressn(3)
      integer index_i(nmax,2),clumax
c ------------------------------------------------------------------

c      write(6,*) 'start CONFIG'
c ***    select a chain at random ***
      rchain  = random()
      do icbu = 1,nmolty
         if ( rchain .lt. pmcbmt(icbu) ) then
            imolty = icbu
            rchain = 2.0d0
         endif
      enddo

      if (ncmt(1,imolty).eq.0) then
         return
      endif
      i = idint( dble(ncmt(1,imolty))*random() ) + 1
      i = parbox(i,1,imolty)
      if ( moltyp(i) .ne. imolty ) write(istdout,*) 'screwup config'
      ibox=1

c *** store number of units in iunit and # to be grown in igrow ***
      iunit = nunit(imolty)
      igrow = nugrow(imolty)
 
c *** store position of trial chain in r x/y/z cbu ***
      do icbu = 1, igrow
         rxnew(icbu) = rxu(i,icbu)
         rynew(icbu) = ryu(i,icbu)
         rznew(icbu) = rzu(i,icbu)
      enddo

      call schedule(igrow,imolty,islen,iutry,0,1)

c     --- determine how many beads are being regrown
      total = 0
      do icbu = 1,igrow
         if ( .not. lexshed(icbu) ) total = total + 1
      enddo

      bncb(imolty,total) = bncb(imolty,total) + 1.0d0
 
c     --- grow new chain conformation
      call rosenbluth ( .true., lterm,i,i,imolty,islen,ibox,igrow )

c --- termination of cbmc attempt due to walk termination ---
      if ( lterm ) then 
c        write(6,*) 'termination of growth'
        return
      endif


c     --- grow old chain conformation
      call rosenbluth ( .false.,lterm,i,i,imolty,islen,ibox,igrow)

c     --- termination of old walk due to problems generating orientations
      if ( lterm ) then
         write(istdout,*) 'CONFIG: old growth rejected'
         return
      endif

C -----------------------------------------------------------------------------
c     Begin DC-CBMC, Explicit Atom and Ewald-sum Corrections 

      if ( ldual .or. iunit .ne. igrow 
     &     .or. ((.not. lchgall) .and. lelect(imolty)) ) then
c     --- Put on hydrogens for explicit AA model for calculation of COM
c     --- and assign all of the grown new and old beads to rxuion
c     --- with old = 1, new = 2
         do j=1,igrow
            rxuion(j,1)=rxu(i,j)
            ryuion(j,1)=ryu(i,j)
            rzuion(j,1)=rzu(i,j)
            qquion(j,1)=qqu(i,j)
         enddo	 
         do j = 1,igrow
            rxuion(j,2) = rxnew(j)
            ryuion(j,2) = rynew(j)
            rzuion(j,2) = rznew(j)
            qquion(j,2) = qquion(j,1)
         enddo
         if ( lmisc ) then
            nchp1 = n1 + 1
         else
            nchp1=nsum+1
         endif
         nboxi(nchp1) = ibox
         moltyp(nchp1) = imolty
         moltion(1) = imolty
         moltion(2) = imolty

         if ( igrow .ne. iunit ) then
c           -- iii = 1 old conformation
            do j = igrow+1, iunit
               rxuion(j,1) = rxu(i,j)
               ryuion(j,1) = ryu(i,j)
               rzuion(j,1) = rzu(i,j)
               qquion(j,1) = qqu(i,j)
            enddo
c           -- iii = 2 new conformation
            do j=1, igrow
               rxu(nchp1,j) = rxnew(j)
               ryu(nchp1,j) = rynew(j)
               rzu(nchp1,j) = rznew(j)
            enddo
            call explct(nchp1,vtornew,.false.,.false.)
            do j=igrow+1, iunit
               rxuion(j,2) = rxu(nchp1,j)
               ryuion(j,2) = ryu(nchp1,j)
               rzuion(j,2) = rzu(nchp1,j)
               qquion(j,2) = qquion(j,1)
            enddo
         else
            vtornew = 0d0
         endif
      endif

      if (ldual .or. ((.not. lchgall) .and. lelect(imolty)) 
     &     .or. igrow .ne. iunit) then
         istt = 1
         iett = iunit

c        -- check new before old
         do iii = 2,1,-1
c          calculate the Full rcut Lennard-Jones energy for the grown beads
c          iii = 1 old conformation
c          iii = 2 new conformation
            if (iii .eq. 1) then
               ltors = .true.
            else
               ltors = .false.
            endif
            
            call energy (i,imolty,v,vintra,vinter,vext,velect,vewald,iii
     &           ,ibox,istt,iett,.true.,ovrlap,ltors,vtorold,.false.)

            if (iii .eq. 2) then
               if (ovrlap) then
                  return
               endif

               delen = v - ( vnewinter + vnewext + vnewelect 
     &              +vnewintra + vnewewald) +vtornew
               if ( delen*beta .gt. (2.3d0*softcut) ) then
                  return
               endif
               weight    = weight*dexp(-(beta*delen))
               vnewt     = vnewt + delen
               vnewinter = vinter
               vnewintra = vintra
               vnewext   = vext
               vnewelect = velect
               vnewtg = vnewtg + vtornew
               vnewewald = vewald

            else
               deleo = v - ( voldinter + voldext + voldelect
     &              + voldintra +voldewald) + vtorold
               weiold    = weiold*dexp(-(beta*deleo))
               voldt     = voldt + deleo
               voldintra = vintra
               voldinter = vinter
               voldext   = vext
               voldtg    = voldtg + vtorold
               voldelect = velect
               voldewald = vewald
            endif
         enddo
         
      endif

c     End of DC-CBMC, Explicit Atom and Ewald-sum Corrections

c *** check for acceptance of trial configuration ***
      wnlog = dlog10 ( weight )
      wolog = dlog10 ( weiold )
      wdlog = wnlog - wolog
      if ( wdlog .lt. -softcut ) then
         return
      endif

      bscb(imolty,1,total) = bscb(imolty,1,total) + 1.0d0
 
      wratio = weight / weiold

      if ( random() .gt. wratio ) then
         return
      endif
      if ( lcluster2 .and. nchbox(ibox) .gt. 1) then
         call cluster(nchbox(ibox),i,ovrlap,.false.)
         if (ovrlap) return
      endif

c         write(6,*) 'CONFIG accepted',i,ibox,wratio
c        --- we can now accept !!!!! ***
         bscb(imolty,2,total) = bscb(imolty,2,total) + 1.0d0
         vbox(ibox)    = vbox(ibox)    + ( vnewt - voldt )
	 vinterb(ibox) = vinterb(ibox) + (vnewinter - voldinter)
	 vintrab(ibox) = vintrab(ibox) + (vnewintra- voldintra)
	 vvibb(ibox)   =  vvibb(ibox)  + (vnewbvib- voldbvib)
	 vtgb(ibox)    = vtgb(ibox)    + (vnewtg- voldtg)
	 vextb(ibox)   = vextb(ibox)   + (vnewext - voldext)
	 vbendb(ibox)  = vbendb(ibox)  + (vnewbb - voldbb)
         velectb(ibox) = velectb(ibox) + (vnewelect - voldelect)
     &        + (vnewewald - voldewald)

         do ic = 1, igrow
            rxu(i,ic) = rxnew(ic)
            ryu(i,ic) = rynew(ic)
            rzu(i,ic) = rznew(ic)
         enddo
         do ic = igrow+1, iunit
            rxu(i,ic)  = rxuion(ic,2)
            ryu(i,ic)  = ryuion(ic,2)
            rzu(i,ic)  = rzuion(ic,2)
         enddo

         call ctrmas(.false.,ibox,i,7)

         if ( lene_w ) then
            do j = 1,nchbox(ibox)
               ene1(j) = ene1(j)+ene3(j,2)-ene3(j,1)
               ene2(j) = ene2(j)+ene4(j,2)-ene4(j,1)
            enddo
         endif

c         if ( lneighbor ) then
            do 10 ic = 1, neigh_cnt(i)
               j = neighbor(ic,i)
               do ip = 1,neigh_cnt(j)
                  if ( neighbor(ip,j) .eq. i ) then
                     neighbor(ip,j)=neighbor(neigh_cnt(j),j)
                     neigh_cnt(j) = neigh_cnt(j)-1
                     goto 10
                  endif
               enddo
 10         continue
            neigh_cnt(i) = neigh_icnt
            do ic = 1,neigh_icnt
               j = neighi(ic)
               neighbor(ic,i)=j
               lneighij = .false.
               do ip = 1,neigh_cnt(j)
                  if ( neighbor(ip,j) .eq. i ) then
                     lneighij = .true.
                  endif
               enddo
               if ( .not. lneighij ) then
                  neigh_cnt(j) = neigh_cnt(j)+1
                  neighbor(neigh_cnt(j),j) = i
               endif
            enddo

            if ( lene_cut ) then
               do 20 ic = 1, neigh_cnt2(i)
                  j = neighbor2(ic,i)
                  do ip = 1,neigh_cnt2(j)
                     if ( neighbor2(ip,j) .eq. i ) then
                        neighbor2(ip,j)=neighbor2(neigh_cnt2(j),j)
                        neigh_cnt2(j) = neigh_cnt2(j)-1
                        goto 20
                     endif
                  enddo
 20            continue
               neigh_cnt2(i) = neigh_icnt2
               do ic = 1,neigh_icnt2
                  j = neighi2(ic)
                  neighbor2(ic,i)=j
                  lneighij = .false.
                  do ip = 1,neigh_cnt2(j)
                     if ( neighbor2(ip,j) .eq. i ) then
                        lneighij = .true.
                     endif
                  enddo
                  if ( .not. lneighij ) then
                     neigh_cnt2(j) = neigh_cnt2(j)+1
                     neighbor2(neigh_cnt2(j),j) = i
                  endif
               enddo
            endif
c         endif

c -----------------------------------------------------------------
c       write(6,*) 'end CONFIG'
      return
      end








