      subroutine rosenbluth ( lnew,lterm,i,icharge,imolty,ifrom,ibox
     &     ,igrow )
 
c    *******************************************************************
c    **   performs a configurational bias move for branched molecules **
c    *******************************************************************

      implicit none
      
      include 'control.inc'
      include 'coord.inc'
      include 'system.inc'
      include 'ensemble.inc'
      include 'poten.inc'
      include 'conver.inc'
      include 'cbmc.inc'
      include 'rosen.inc' 
      include 'connect.inc'
      include 'parallel.inc'

c     --- variables passed to the subroutine
      logical lnew,lterm
      integer i,icharge,imolty,ifrom,ibox,igrow

c     --- local variables
      
      logical ovrlap,ltorsion

      integer glist,iuprev,iufrom,ichoi,ntogrow,count
      integer iu,iv,iw,ju,ip,ichtor
     &       ,it,jut2,jut3,jut4,jttor,iwalk,ivect
      integer angstart
      
      dimension glist(numax)

      double precision dum,xub,yub,zub,length
      double precision xvec(numax,numax),yvec(numax,numax)
     &                ,zvec(numax,numax),distij(numax,numax)
      double precision vdha,x,y,z,maxlen
     &                ,xaa1,yaa1,zaa1,daa1,xa1a2,ya1a2,za1a2,da1a2
     &                ,thetac,dot,rbf,bsum,bs,random
      double precision vbbtr,vvibtr,vtgtr,vtorso
      double precision bondlen,bendang,phi,phidisp,phinew,thetanew
      double precision xx,yy,zz

      dimension vtgtr(nchmax)
      dimension bondlen(numax),bendang(numax),phi(numax)
      dimension xx(numax),yy(numax),zz(numax)

c     -- new stuff
      integer itor
      double precision bf_tor,bsum_tor,vtorsion,phitors,ran_tor
     &     ,wei_bend
      dimension bf_tor(nchtor_max),vtorsion(nchtor_max)
     &     ,phitors(nchtor_max)
      dimension bsum_tor(nchmax)


c ------------------------------------------------------------------

c      write(6,*) 'start ROSENBLUTH'
      lterm = .false.

c *******************************************
c * Rosenbluth weight of trial conformation *
c *******************************************

c        --- initialize conformation energies and weight
      if ( lnew ) then
c        --- set the initial weight to unity ***
         weight = 1.0d0
c        --- set total energy of trial configuration to zero ***
         vnewt     = 0.0d0
         vnewtg    = 0.0d0
         vnewbb    = 0.0d0
         vnewbvib  = 0.0d0
         vnewext   = 0.0d0
         vnewintra = 0.0d0
         vnewinter = 0.0d0
         vnewelect = 0.0d0
         vnewewald = 0.0d0
      else
c        --- old conformation
c        --- set the initial weight of the old configuration to unity ***
         weiold = 1.0d0
c        --- set total energy of trial configuration to zero ***
         voldt     = 0.0d0
         voldtg    = 0.0d0
         voldbb    = 0.0d0
         voldbvib  = 0.0d0
         voldext   = 0.0d0
         voldintra = 0.0d0
         voldinter = 0.0d0
         voldelect = 0.0d0
         voldewald = 0.0d0
      endif

c     --- set lexist to lexshed
      do iu = 1,igrow
         lexist(iu) = lexshed(iu)
      enddo

c     --- calculate all bond vectors for lexist 
      do iu = 1, igrow
         do iv = 1, invib(imolty,iu)
            ju = ijvib(imolty,iu,iv)
            if ( lexist(iu) .and. lexist(ju) ) then
               if ( lnew ) then
c                 --- use new coordinates
                  xvec(iu,ju) = rxnew(ju) - rxnew(iu)
                  yvec(iu,ju) = rynew(ju) - rynew(iu)
                  zvec(iu,ju) = rznew(ju) - rznew(iu)
               else
c                 --- use old coordinates
                  xvec(iu,ju) = rxu(i,ju) - rxu(i,iu)
                  yvec(iu,ju) = ryu(i,ju) - ryu(i,iu)
                  zvec(iu,ju) = rzu(i,ju) - rzu(i,iu)
               endif
               distij(iu,ju) = dsqrt( xvec(iu,ju)**2
     +              + yvec(iu,ju)**2 + zvec(iu,ju)**2 )
            endif
         enddo
      enddo 

c *************************
c * loop over trial units *
c *************************
      do 200 iw = 1, ifrom
c        --- set vibration and bending energies for this growth to 0.0

         iufrom = growfrom(iw)
         ntogrow = grownum(iw)

c        --- perform the biased selection of bond angles and get lengths
         call geometry(lnew,iw,i,imolty,angstart,iuprev,glist
     &        ,bondlen,bendang,phi,vvibtr,vbbtr,maxlen,wei_bend )

c         write(6,*) 'lnew, wei_bend',lnew,wei_bend

c        --- we now have the bond lengths and angles for the grown beads
c        -- select nchoi trial positions based only on torsions
         ichoi = nchoi(imolty)
         ichtor = nchtor(imolty)
         do ip = 1,ichoi
            ivect = 0

c           --- set up the cone based on iuprev (could be grown if no prev)
            if ( growprev(iw) .eq. 0 ) then
c              --- calculate random vector on the unit sphere for the first bead
               count = 1
               if ( (.not. lnew) .and. ip .eq. 1 ) then
c                 --- use old molecule position
                  iu = growlist(iw,count)
                  length = bondlen(count)
c                 --- compute unit vector to be used in cone and torsion
                  x = ( rxu(i,iu) - rxu(i,iufrom) )/length 
                  y = ( ryu(i,iu) - ryu(i,iufrom) )/length 
                  z = ( rzu(i,iu) - rzu(i,iufrom) )/length 
c                 --- store this in xx yy zz
                  xx(count) = x
                  yy(count) = y
                  zz(count) = z
               else
c                 --- choose randomly on the unit sphere
                  call sphere(x,y,z)
                  xx(count) = x
                  yy(count) = y
                  zz(count) = z
               endif

               if ( ntogrow .gt. 1 ) then
c                 ---set up the cone 
                  xub = -x 
                  yub = -y 
                  zub = -z 
                  call cone (1,xub,yub,zub,dum,dum,dum,dum,dum )
               endif
               ltorsion = .false.
            else
c              --- set up the cone based on iuprev and iufrom
               length = distij(iuprev,iufrom)
               xub = xvec(iuprev,iufrom) / length 
               yub = yvec(iuprev,iufrom) / length
               zub = zvec(iuprev,iufrom) / length

               call cone (1,xub,yub,zub,dum,dum,dum,dum,dum )
               ltorsion = .true.
            endif

c           --- Begin loop to determine torsional angle

            if ( ltorsion ) then

c              --- initialize bsum_tor
               bsum_tor(ip) = 0.0d0
               
               do itor = 1,ichtor

                  if ( (.not. lnew) .and. ip .eq. 1 
     &                 .and. itor .eq. 1) then
c                    --- old conformation - set phidisp to 0.0d0
                     phidisp = 0.0d0
                  else
c                    --- choose a random displacement angle from anglestart
c                    --- assign the positions based on angles and lengths above
                     phidisp = twopi*random()
                  endif

                  do count = angstart,ntogrow
                     phinew = phi(count) + phidisp
                     thetanew = bendang(count)
                     
                     call cone(2,dum,dum,dum,thetanew,phinew,x,y,z)
c     --- store the unit vectors in xx, yy, zz
                     xx(count) = x
                     yy(count) = y
                     zz(count) = z
                  enddo

c              --- set energies of trial position to zero ---
                  vdha = 0.0d0
c              --- compute torsion energy for given trial conformation
                  do count = 1,ntogrow
                     iu = growlist(iw,count)
                     do it = 1, intor(imolty,iu)
                        jut2 = ijtor2(imolty,iu,it)
                        jut3 = ijtor3(imolty,iu,it)
                        if ( jut2 .eq. iufrom .and. 
     &                       jut3 .eq. iuprev) then
                           jut4 = ijtor4(imolty,iu,it)
                           
c                       --- jut4 must already exist or we made a big mistake
                           if ( .not. lexist(jut4) )  then
                              write(istdout,*) 'jut4,jut3,jut2,iu',
     &                             jut4,jut3,jut2,iu
                              stop 'trouble jut4'
                           endif
                           jttor = ittor(imolty,iu,it)
                           
c                       --- calculate cross products d_a x d_a-1 
                           xaa1 = yy(count) * zvec(jut3,jut2) 
     &                          + zz(count) * yvec(jut2,jut3)
                           yaa1 = zz(count) * xvec(jut3,jut2) 
     &                          + xx(count) * zvec(jut2,jut3)
                           zaa1 = xx(count) * yvec(jut3,jut2) 
     &                          + yy(count) * xvec(jut2,jut3)
                           
c                       --- calculate cross products d_a-1 x d_a-2
                           xa1a2 = yvec(jut2,jut3) * zvec(jut3,jut4) -
     +                          zvec(jut2,jut3) * yvec(jut3,jut4)
                           ya1a2 = zvec(jut2,jut3) * xvec(jut3,jut4) -
     +                          xvec(jut2,jut3) * zvec(jut3,jut4)
                           za1a2 = xvec(jut2,jut3) * yvec(jut3,jut4) -
     +                          yvec(jut2,jut3) * xvec(jut3,jut4)

c                       --- calculate lengths of cross products ***
                           daa1 = dsqrt ( xaa1**2 + yaa1**2 + zaa1**2 )
                           da1a2 = dsqrt ( xa1a2**2 + ya1a2**2 
     &                          + za1a2**2 )
                           
c                       --- calculate dot product of cross products ***
                           dot = xaa1*xa1a2 + yaa1*ya1a2 + zaa1*za1a2
                           thetac = - (dot / ( daa1 * da1a2 ))
                        
c                       --- add torsion energy to vdha
                           vdha = vdha + vtorso( thetac, jttor )
                        endif
                     enddo
                  enddo

c                 --- compute boltzmann factor and add it to bsum_tor
                  bf_tor(itor) = dexp ( -vdha * beta )
                  bsum_tor(ip) = bsum_tor(ip) + bf_tor(itor)

c                 --- store vtorsion and phidisp for this trial
                  vtorsion(itor) = vdha
                  phitors(itor) = phidisp

               enddo

               if ( lnew .or. ip .ne. 1 ) then
c                 --- choose one of the trial sites in a biased fashion
                  ran_tor = random()*bsum_tor(ip)
                  bs = 0.0d0
                  do itor = 1,ichtor
                     bs = bs + bf_tor(itor)
                     if ( ran_tor .lt. bs ) then
c                       --- save torsion energy of this trial position
                        vtgtr(ip) = vtorsion(itor)

c                       --- assign the phidisp of this trial postion
                        phidisp = phitors(itor)

c                       --- exit the loop
                        goto 100
                     endif
                  enddo
 100              continue
               else
c                 --- select the old conformation
                  vtgtr(ip) = vtorsion(1)
                  phidisp = phitors(1)
               endif

c              --- divide bsum by ichtor
               bsum_tor(ip) = bsum_tor(ip) / dble(ichtor)
               
            else

c              --- no torsion energy, choose phidisp at random (except old)
               if ( (.not. lnew) .and. ip .eq. 1 ) then
c                 --- old conformation - set phidisp to 0.0d0
                  phidisp = 0.0d0
               else

c                 --- choose a random displacement angle from anglestart
c                 --- assign the positions based on angles and lengths above
                  phidisp = twopi*random()
               endif

c              --- assign the torsional energy a value of 0.0
               vtgtr(ip) = 0.0d0

c              --- set bsum_tor to 1.0d0
               bsum_tor(ip) = 1.0d0

            endif

c           --- for accepted phidisp set up the vectors
            do count = angstart,ntogrow
               phinew = phi(count) + phidisp
               thetanew = bendang(count)
               
               call cone(2,dum,dum,dum,thetanew,phinew,x,y,z)
c              --- store the unit vectors in xx, yy, zz
               xx(count) = x
               yy(count) = y
               zz(count) = z
            enddo

c           --- accepted coordinates, save them in r*p(trial)
            do count = 1,ntogrow
               length = bondlen(count)
               if ( lnew ) then
c                 --- use new positions
                  rxp(count,ip) = rxnew(iufrom) + xx(count)*length
                  ryp(count,ip) = rynew(iufrom) + yy(count)*length
                  rzp(count,ip) = rznew(iufrom) + zz(count)*length
               else
c                 --- use old coordinates
                  rxp(count,ip) = rxu(i,iufrom) + xx(count)*length
                  ryp(count,ip) = ryu(i,iufrom) + yy(count)*length
                  rzp(count,ip) = rzu(i,iufrom) + zz(count)*length
               endif
            enddo

         enddo

c       --- now that we have the trial site need to compute non-bonded energy

         call boltz ( lnew,.false.,.false.,ovrlap,i,icharge,imolty,ibox
     &        ,ichoi,iufrom ,ntogrow, glist, maxlen)

         if ( ovrlap ) then
            lterm = .true.
            return
         endif

c ---------------------------------------------------------------------
 
c *** perform the walk according to the availibility of the choices ***
c *** and calculate the correct weight for the trial walk           ***

         bsum = 0.0d0
         do ip = 1, ichoi
c           --- include both the torsional and the LJ/qq
            bsum = bsum + bfac(ip)*bsum_tor(ip)
         enddo

         if ( lnew ) then
c           --- update new rosenbluth weight - include bending weight
            weight = weight * bsum * wei_bend

            if ( weight .lt. softlog ) then
               lterm=.true.
               return
            endif

c           --- select one position at random ---
            rbf = bsum * random()
            bs = 0.0d0 
            do 170 ip = 1, ichoi
               if ( .not. lovr(ip) ) then
                  bs = bs + bfac(ip)*bsum_tor(ip)
                  if ( rbf .lt. bs ) then
c                    --- select ip position ---
                     iwalk = ip
                     goto 180
                  endif
               endif
 170        continue
         else
c           --- old conformation, update weiold - include wei_bend
            weiold = weiold * bsum * wei_bend
            if (weiold .lt. softlog) write(istdout,*) 
     &           '###old weight too low'
         endif

 180     if ( lnew ) then
c           --- update new trial energies
            vnewt     = vnewt     + vtry(iwalk)  
     +           + vtgtr(iwalk) + vvibtr + vbbtr
            vnewbvib  = vnewbvib  + vvibtr
            vnewbb    = vnewbb    + vbbtr
            vnewtg    = vnewtg    + vtgtr(iwalk)
            vnewext   = vnewext   + vtrext(iwalk)
            vnewintra = vnewintra + vtrintra(iwalk)
            vnewinter = vnewinter + vtrinter(iwalk)
            vnewelect = vnewelect + vtrelect(iwalk)
            vnewewald = vnewewald + vtrewald(iwalk)
         else
c            --- update old trail energies
            voldt     = voldt     + vtry(1)  
     +           + vtgtr(1) + vvibtr + vbbtr
            voldbvib  = voldbvib  + vvibtr
            voldbb    = voldbb    + vbbtr
            voldtg    = voldtg    + vtgtr(1)
            voldext   = voldext   + vtrext(1)
            voldintra = voldintra + vtrintra(1)
            voldinter = voldinter + vtrinter(1)
            voldelect = voldelect + vtrelect(1)
            
            voldewald = voldewald + vtrewald(1)
         endif

         do count = 1,ntogrow
            iu = growlist(iw,count)
            if ( lnew ) then
c              --- assign new positions to r*new
               rxnew(iu) = rxp(count,iwalk)
               rynew(iu) = ryp(count,iwalk)
               rznew(iu) = rzp(count,iwalk)
            endif

c           --- set lexist(iu) to true so ewald sum computed properly
            lexist(iu) = .true.

c           --- store new existing vectors between beads and iufrom
            if ( lnew ) then
c              --- use r*new positions
               xvec(iu,iufrom) = rxnew(iufrom) - rxnew(iu)
               yvec(iu,iufrom) = rynew(iufrom) - rynew(iu)
               zvec(iu,iufrom) = rznew(iufrom) - rznew(iu)
            else
c              --- use r*u positions
               xvec(iu,iufrom) = rxu(i,iufrom) - rxu(i,iu)
               yvec(iu,iufrom) = ryu(i,iufrom) - ryu(i,iu)
               zvec(iu,iufrom) = rzu(i,iufrom) - rzu(i,iu)
            endif
            distij(iu,iufrom) = dsqrt( xvec(iu,iufrom)**2
     +           + yvec(iu,iufrom)**2 + zvec(iu,iufrom)**2 )
            
            xvec(iufrom,iu) = - xvec(iu,iufrom)
            yvec(iufrom,iu) = - yvec(iu,iufrom)
            zvec(iufrom,iu) = - zvec(iu,iufrom)
            distij(iufrom,iu) = distij(iu,iufrom)
         enddo

c ********************************
c * end of loop over trial units *
c ********************************
 200  continue

c      write(6,*) 'end ROSENBLUTH'

c ------------------------------------------------------------------

      return
      end


