      subroutine rotxyz (lx,ly,lz )
     
c    *******************************************************************
c    ** makes a rotational movement around "x" space-fixed axis.      **
c    ** the maximum displacement is controlled by rmrotx and the      **
c    ** number of successful rotation is given by bsrotx.             **
c    **                                                               **
c    ** rotxyz chooses one of the three space-fixed axes at random    **
c    ** and rotates the molecule around this axis by dgamma radians.  **
c    ** the maximum angular displacement is dgamax.                   **
c    *******************************************************************
      implicit none

c *** common blocks ***
      include 'control.inc'
      include 'coord.inc'
      include 'coord2.inc'
      include 'ensemble.inc'
      include 'system.inc' 
      include 'inputdata.inc'
      include 'bnbsma.inc'
      include 'neigh.inc'
      include 'parallel.inc'

      logical lx,ly,lz,ovrlap,lneighij,lclu_cmp,lexclude(nmax)
      integer i,ibox,flagon,iunit,j,imolty,iuroty,icbu,ic,ip,k,imol
      double precision rx,ry,rz,dchain,rchain,random,vnew,vold
     +                ,vintrao,dgamma,rxorig,ryorig,rzorig
     +                ,rxnew2,rynew2,rznew2,vintran,disvsq,deltv
     +                ,deltvb,vintern,vintero,vextn,vexto,vdum
     &                ,velectn,velecto
c *** further variable definitions
      double precision cosdg, sindg, rmrot

      logical laccept
C --------------------------------------------------------------------

c      write(6,*) 'start ROTXYZ'
      ovrlap = .false.

      ibox = 1
      rchain  = random()
      do icbu = 1,nmolty
         if ( rchain .lt. pmromt(icbu) ) then
            imolty = icbu
            rchain = 2.0d0
         endif
      enddo
      if (ncmt(ibox,imolty).eq.0) then
         if(lx) bnrotx(imolty,ibox) = bnrotx(imolty,ibox) + 1.0d0
         if(ly) bnroty(imolty,ibox) = bnroty(imolty,ibox) + 1.0d0
         if(lz) bnrotz(imolty,ibox) = bnrotz(imolty,ibox) + 1.0d0
         return
      endif

 10   i = idint( dble(ncmt(1,imolty))*random() ) + 1
      i = parbox(i,1,imolty)
      if ( moltyp(i) .ne. imolty ) write(istdout,*) 
     &     i,moltyp(i),imolty,'screwup rotxyz'
      if ( lcluster1 .and. i .eq. 1 ) then
         if ( ncmt(1,imolty) .eq. 1 ) return
         goto 10
      endif

      iuroty = iurot(imolty)

c *** store number of units of i in iunit ***
      iunit = nunit(imolty)

c *** store current positions in old-new array #1# ***
      do j = 1, iunit
         rxuion(j,1) = rxu(i,j)
         ryuion(j,1) = ryu(i,j)
         rzuion(j,1) = rzu(i,j)
         qquion(j,1) = qqu(i,j)
      enddo

      moltion(1) = imolty
 
c *** choose a random angular displacement ***
 
      if (lx) then
         rmrot = rmrotx(imolty,ibox)
         bnrotx(imolty,ibox) = bnrotx(imolty,ibox) + 1.0d0
      else if (ly) then
         rmrot = rmroty(imolty,ibox)
         bnroty(imolty,ibox) = bnroty(imolty,ibox) + 1.0d0
      else if (lz) then
         rmrot = rmrotz(imolty,ibox)
         bnrotz(imolty,ibox) = bnrotz(imolty,ibox) + 1.0d0
      endif
      dgamma = ( 2.0d0*random() - 1.0d0 ) * rmrot
 
c *** set up the rotation marix ***
 
      cosdg = dcos( dgamma )
      sindg = dsin( dgamma )
 
c *** Determine the rotation coordinates ***
      if (iuroty .eq. 0) then
c *** Use the center of mass for rotation
         rxorig = xcm(i)
         ryorig = ycm(i)
         rzorig = zcm(i)
      else
c *** Use iurot for rotation
         rxorig = rxuion(iuroty,1)
         ryorig = ryuion(iuroty,1)
         rzorig = rzuion(iuroty,1)
      endif

      if (lx) then 
 
C ***    ROTATE UNITS OF I AROUND X-AXIS ***
 
         do  j = 1, iunit
            ry = ryuion(j,1) - ryorig
            rz = rzuion(j,1) - rzorig
            rynew2 = cosdg * ry + sindg * rz
            rznew2 = cosdg * rz - sindg * ry
               
            rxuion(j,2) = rxuion(j,1)
            ryuion(j,2) = ryorig + rynew2
            rzuion(j,2) = rzorig + rznew2
            qquion(j,2) = qquion(j,1)
         enddo
      endif
      if (ly) then 
 
C ***    ROTATE UNITS OF I AROUND y-AXIS ***
 
         do  j = 1, iunit
            rx = rxuion(j,1) - rxorig
            rz = rzuion(j,1) - rzorig
            rxnew2 = cosdg * rx - sindg * rz
            rznew2 = cosdg * rz + sindg * rx

            rxuion(j,2) = rxorig + rxnew2
            ryuion(j,2) = ryuion(j,1)
            rzuion(j,2) = rzorig + rznew2
            qquion(j,2) = qquion(j,1)
         enddo

      endif
      if (lz) then 
 
C ***    ROTATE UNITS OF I AROUND z-AXIS ***
 
         do  j = 1, iunit
            rx = rxuion(j,1) - rxorig
            ry = ryuion(j,1) - ryorig
            rxnew2 = cosdg * rx + sindg * ry
            rynew2 = cosdg * ry - sindg * rx

            rxuion(j,2) = rxorig + rxnew2
            ryuion(j,2) = ryorig + rynew2
            rzuion(j,2) = rzuion(j,1)
            qquion(j,2) = qquion(j,1)
         enddo

      endif 
      moltion(2) = imolty
 
c  *** calculate the energy of i in the new configuration ***
      
      flagon = 2

      call energy(i,imolty, vnew,vintran,vintern,vextn,velectn,vdum
     &     ,flagon, ibox,1,iunit,.false.,ovrlap,.false.,vdum,.false.)

      if (ovrlap) then
         return
      endif
 
c *** calculate the energy of i in the old configuration ***
      flagon = 1

      call energy(i,imolty, vold,vintrao,vintero,vexto,velecto,vdum
     &     ,flagon,ibox, 1, iunit,.false.,ovrlap,.false.,vdum,.false.)

      if (ovrlap) then
         write(istdout,*) 'disaster- overlap for old conf in ROTXYZ'
         stop
      endif

c *** check for acceptance ***
 
      deltv  = vnew - vold
      deltvb = beta * deltv

c *** For ANES algorithm, do the Fluctuating charge moves.

c      if ( lanes ) then
c         call anes(i,ibox,ibox,2,laccept,deltv,vintern,vintran,vextn,
c     &        velectn,vintero,vintrao,vexto,velecto,vdum,vdum,vdum,
c     &        vdum,.false.)
c         if (laccept) then
c            if (lx) bsrotx(imolty,ibox) = bsrotx(imolty,ibox) + 1.0d0
c            if (ly) bsroty(imolty,ibox) = bsroty(imolty,ibox) + 1.0d0
c            if (lz) bsrotz(imolty,ibox) = bsrotz(imolty,ibox) + 1.0d0
c         endif
c         return         
c      endif

      if ( deltvb .gt. (2.3d0*softcut) ) then
         return
      endif

      if ( deltv .le. 0.0d0 ) then
c        --- move accepted
      elseif ( dexp(-deltvb) .gt. random() ) then
c        --- move accepted
      else
c        --- move rejected
         return
      endif

      if ( lcluster2 .and. nchbox(ibox) .gt. 1 ) then
         call cluster(nchbox(ibox),i,ovrlap,.false.)
         if (ovrlap) return
      endif     

c      write(6,*) 'ROTXYZ accepted',i,deltv

      vbox(ibox) = vbox(ibox) + deltv
      vinterb(ibox)  = vinterb(ibox) + (vintern -  vintero)
      vintrab(ibox)  = vintrab(ibox) + (vintran - vintrao)
      vextb(ibox)    = vextb(ibox)   + (vextn-vexto)
      velectb(ibox)  = velectb(ibox) + (velectn-velecto)

      do j = 1, iunit
         rxu(i,j) = rxuion(j,2)
         ryu(i,j) = ryuion(j,2)
         rzu(i,j) = rzuion(j,2)
      enddo

c *** update center of mass
      if ( iuroty .eq. 0 ) then
         call ctrmas(.false.,ibox,i,2)
      else
         call ctrmas(.false.,ibox,i,7)
      endif

      if (lx) bsrotx(imolty,ibox) = bsrotx(imolty,ibox) + 1.0d0
      if (ly) bsroty(imolty,ibox) = bsroty(imolty,ibox) + 1.0d0
      if (lz) bsrotz(imolty,ibox) = bsrotz(imolty,ibox) + 1.0d0

      if ( lene_w ) then
         do j = 1,nchbox(ibox)
            ene1(j) = ene1(j)+ene3(j,2)-ene3(j,1)
            ene2(j) = ene2(j)+ene4(j,2)-ene4(j,1)
         enddo
      endif

      if ( iuroty .ne. 1 ) then
c      if ( lneighbor .and. iuroty .ne. 1 ) then
         do 11 ic = 1, neigh_cnt(i)
            j = neighbor(ic,i)
            do ip = 1,neigh_cnt(j)
               if ( neighbor(ip,j) .eq. i ) then
                  neighbor(ip,j)=neighbor(neigh_cnt(j),j)
                  neigh_cnt(j) = neigh_cnt(j)-1
                  goto 11
               endif
            enddo
 11      continue
         neigh_cnt(i) = neigh_icnt
         do ic = 1,neigh_icnt
            j = neighi(ic)
            neighbor(ic,i)=j
            lneighij = .false.
            do ip = 1,neigh_cnt(j)
               if ( neighbor(ip,j) .eq. i ) then
                  lneighij = .true.
               endif
            enddo
            if ( .not. lneighij ) then
               neigh_cnt(j) = neigh_cnt(j)+1
               neighbor(neigh_cnt(j),j) = i
            endif
         enddo
      endif
      if ( lene_cut ) then
c      if ( lneighbor .and. lene_cut ) then
         do 22 ic = 1, neigh_cnt2(i)
            j = neighbor2(ic,i)
            do ip = 1,neigh_cnt2(j)
               if ( neighbor2(ip,j) .eq. i ) then
                  neighbor2(ip,j)=neighbor2(neigh_cnt2(j),j)
                  neigh_cnt2(j) = neigh_cnt2(j)-1
                  goto 22
               endif
            enddo
 22      continue
         neigh_cnt2(i) = neigh_icnt2
         do ic = 1,neigh_icnt2
            j = neighi2(ic)
            neighbor2(ic,i)=j
            lneighij = .false.
            do ip = 1,neigh_cnt2(j)
               if ( neighbor2(ip,j) .eq. i ) then
                  lneighij = .true.
               endif
            enddo
            if ( .not. lneighij ) then
               neigh_cnt2(j) = neigh_cnt2(j)+1
               neighbor2(neigh_cnt2(j),j) = i
            endif
         enddo
      endif
         
c      write(6,*) 'end ROTXYZ'

      return
      end

