      subroutine traxyz (lx,ly,lz)
 
c    *******************************************************************
c    ** makes a translational movement in x,y,or z-direction.         **
c    ** the maximum displacement is controlled by rmtrax(yz) and the  **
c    ** number of successful trial moves is stored in bstrax(yz).     **
c    ** The attepmts are stored in bntrax(yz)                         **
c    *******************************************************************
 
      implicit none

      include 'control.inc'
      include 'coord.inc'
      include 'coord2.inc'
      include 'ensemble.inc'
      include 'system.inc' 
      include 'inputdata.inc'
      include 'bnbsma.inc'
      include 'neigh.inc'
      include 'parallel.inc'

      logical lx,ly,lz,ovrlap,lneighij,lclu_cmp,lexclude(nmax)
      integer i,ibox,flagon,iunit,j,imolty,icbu,ncount,ic,ip,k,imol
      double precision rx,ry,rz,dchain,ddx,ddy,ddz,random,vnew,vold
     +                 ,vintran,vintrao,deltv,deltvb,disvsq
     +                 ,vintern,vintero,vextn,vexto,rchain
     &                 ,velectn,velecto,vdum
      
      logical laccept
C --------------------------------------------------------------------

c      write(6,*) 'start TRAXYZ'
      ovrlap = .false.
c     ***    select a chain at random ***
      rchain  = random()
      do icbu = 1,nmolty
         if ( rchain .lt. pmtrmt(icbu) ) then
            imolty = icbu
            rchain = 2.0d0
         endif
      enddo
      
      ibox = 1
      if (ncmt(ibox,imolty).eq.0) then
         if(lx) bntrax(imolty,ibox) = bntrax(imolty,ibox) + 1.0d0
         if(ly) bntray(imolty,ibox) = bntray(imolty,ibox) + 1.0d0
         if(lz) bntraz(imolty,ibox) = bntraz(imolty,ibox) + 1.0d0
         return
      endif
 11   i = idint( dble(ncmt(1,imolty))*random() ) + 1
      i = parbox(i,1,imolty)
      if ( moltyp(i) .ne. imolty ) write(istdout,*) 
     &     moltyp(i),i,imolty,'screwup traxyz'
      if ( lcluster1 .and. i .eq. 1 ) then
         if ( ncmt(1,imolty) .eq. 1 ) return
         goto 11
      endif
         
c *** store number of units of i in iunit ***

      iunit = nunit(imolty)

      do  j = 1, iunit
         rxuion(j,1) = rxu(i,j)
         ryuion(j,1) = ryu(i,j)
         rzuion(j,1) = rzu(i,j)
         qquion(j,1) = qqu(i,j)
      enddo
      moltion(1) = imolty

c *** move i ***

      if (lx) then
         rx =  ( 2.0*random() - 1.0d0 ) * rmtrax(imolty,ibox)
         bntrax(imolty,ibox) = bntrax(imolty,ibox) + 1.0d0
      else
         rx=0
      endif
      if (ly) then
         ry =  ( 2.0*random() - 1.0d0 ) * rmtray(imolty,ibox)
         bntray(imolty,ibox) = bntray(imolty,ibox) + 1.0d0
      else
         ry=0
      endif
      if (lz) then
         rz =  ( 2.0*random() - 1.0d0 ) * rmtraz(imolty,ibox)
         bntraz(imolty,ibox) = bntraz(imolty,ibox) + 1.0d0
      else
         rz=0
      endif
      do j = 1,iunit
         rxuion(j,2) = rxuion(j,1) + rx 
         ryuion(j,2) = ryuion(j,1) + ry 
         rzuion(j,2) = rzuion(j,1) + rz 
         qquion(j,2) = qquion(j,1)
      enddo

      if ( lcluster1 .and. (rxuion(1,2)-rxu(1,1))**2+
     &     (ryuion(1,2)-ryu(1,1))**2+(rzuion(1,2)
     &     -rzu(1,1))**2 .gt. 
     &     (rbmax(imolty)+rbmax(moltyp(1)))**2) return
      moltion(2) = imolty

c  *** calculate the energy of i in the new configuration ***
      flagon = 2
      call energy(i,imolty, vnew,vintran, vintern,vextn,velectn
     &     ,vdum,flagon, ibox,1, iunit,.false.,ovrlap,.false.
     &     ,vdum,.false.)

      if (ovrlap) then
         return
      endif

c *** calculate the energy of i in the old configuration ***
      flagon = 1
      call energy(i,imolty,vold,vintrao,vintero,vexto,velecto
     &     ,vdum,flagon,ibox,1, iunit,.false.,ovrlap,.false.
     &     ,vdum,.false.)

      if (ovrlap) then
         write(istdout,*) 'disaster ovrlap in old conf of TRAXYZ'
         stop
      endif

c *** check for acceptance ***
 
      deltv  = vnew - vold
      deltvb = beta * deltv

c *** For ANES algorithm, do the Fluctuating charge moves.

c      if ( lanes ) then
c         call anes(i,ibox,ibox,1,laccept,deltv,vintern,vintran,vextn,
c     &        velectn,vintero,vintrao,vexto,velecto,vdum,vdum,
c     &        vdum,vdum,.false.)
c         if ( laccept ) then
c            if (lx) bstrax(imolty,ibox) = bstrax(imolty,ibox) + 1.0d0
c            if (ly) bstray(imolty,ibox) = bstray(imolty,ibox) + 1.0d0
c            if (lz) bstraz(imolty,ibox) = bstraz(imolty,ibox) + 1.0d0
c         endif
c         return         
c      endif
 
      if ( deltvb .gt. (2.3d0*softcut) ) then
         return
      endif

      if ( deltv .le. 0.0d0 ) then
c        --- accept move
      elseif ( dexp(-deltvb) .gt. random() ) then
c        --- accept move
      else
c        --- move rejected
         return
      endif

      if ( lcluster2 .and. nchbox(ibox) .gt. 1) then
         call cluster(nchbox(ibox),i,ovrlap,.false.)
         if (ovrlap) return
      endif

c      write(6,*) 'TRAXYZ accepted',i,deltv
               
      vbox(ibox)     = vbox(ibox) + deltv
      vinterb(ibox)  = vinterb(ibox) + (vintern - vintero)
      vintrab(ibox)  = vintrab(ibox) + (vintran - vintrao)
      vextb(ibox)    = vextb(ibox)   + (vextn   - vexto)
      velectb(ibox)   = velectb(ibox)  + (velectn - velecto)
      
      do j = 1,iunit
         if (lx) rxu(i,j) = rxuion(j,2)
         if (ly) ryu(i,j) = ryuion(j,2)
         if (lz) rzu(i,j) = rzuion(j,2)
      enddo

      call ctrmas(.false.,ibox,i,1)
      if (lradgyr) then
         xclust = xclust + (rx + ddx)/nchbox(1)
         yclust = yclust + (ry + ddy)/nchbox(1)
         zclust = zclust + (rz + ddz)/nchbox(1)
      endif
      if (lx) bstrax(imolty,ibox) = bstrax(imolty,ibox) + 1.0d0
      if (ly) bstray(imolty,ibox) = bstray(imolty,ibox) + 1.0d0
      if (lz) bstraz(imolty,ibox) = bstraz(imolty,ibox) + 1.0d0
         
      if ( lene_w ) then
         do j = 1,nchbox(ibox)
            ene1(j) = ene1(j)+ene3(j,2)-ene3(j,1)
            ene2(j) = ene2(j)+ene4(j,2)-ene4(j,1)
         enddo
      endif

c      if ( lneighbor ) then
         
         do 10 ic = 1, neigh_cnt(i)
            j = neighbor(ic,i)
            do ip = 1,neigh_cnt(j)
               if ( neighbor(ip,j) .eq. i ) then
                  neighbor(ip,j)=neighbor(neigh_cnt(j),j)
                  neigh_cnt(j) = neigh_cnt(j)-1
                  goto 10
               endif
            enddo
 10      continue
         neigh_cnt(i) = neigh_icnt
         do ic = 1,neigh_icnt
            j = neighi(ic)
            neighbor(ic,i)=j
            lneighij = .false.
            do ip = 1,neigh_cnt(j)
               if ( neighbor(ip,j) .eq. i ) then
                  lneighij = .true.
               endif
            enddo
            if ( .not. lneighij ) then
               neigh_cnt(j) = neigh_cnt(j)+1
               neighbor(neigh_cnt(j),j) = i
            endif
         enddo
         if ( lene_cut ) then
            do 20 ic = 1, neigh_cnt2(i)
               j = neighbor2(ic,i)
               do ip = 1,neigh_cnt2(j)
                  if ( neighbor2(ip,j) .eq. i ) then
                     neighbor2(ip,j)=neighbor2(neigh_cnt2(j),j)
                     neigh_cnt2(j) = neigh_cnt2(j)-1
                     goto 20
                  endif
               enddo
 20         continue
            neigh_cnt2(i) = neigh_icnt2
            do ic = 1,neigh_icnt2
               j = neighi2(ic)
               neighbor2(ic,i)=j
               lneighij = .false.
               do ip = 1,neigh_cnt2(j)
                  if ( neighbor2(ip,j) .eq. i ) then
                     lneighij = .true.
                  endif
               enddo
               if ( .not. lneighij ) then
                  neigh_cnt2(j) = neigh_cnt2(j)+1
                  neighbor2(neigh_cnt2(j),j) = i
               endif
            enddo
         endif
c      endif

c      write(6,*) 'end TRAXYZ',i

      return
      end



